function [T_upt] = T_update(T_prev, S, F, log_filt_prev, w1, w2)

% [T_upt, log_T_upt] = T_update(T_prev, S, F, log_filt_prev, gam)
%
% This function updates the intermediate function T of the SMC-FS
% implementation of on-line EM, developed by (Del Moral et al., 2009).
% Here, T_prev is the previous intermediate function, S is the matrix of
% additive component of the sufficient statistics for the current time such
% that S(i, j) = s(x_(t-1)^(i), x_(t)^(j)), F is the log-state transition 
% matrix such that F(i, j) = p(x_(t)^(i) | x_(t-1)^(j)), filt_prev is the
% filtering density for the previous time such that 
% filt_prev(j) = p(x_(t-1)^(j)), gam is the stepsize at the current time.
%

N = size(F, 1);
log_filt_prev = log_filt_prev(:); 

T_upt = zeros(N,1);

for n = 1:N
    m = log_sum(log_filt_prev + F(n,:)');
    if nargin == 6
        T_upt(n) = exp(log_filt_prev' + F(n,:) - m )*(w1*S(:,n) + w2*T_prev);
    elseif nargin == 5
        T_upt(n) = exp(log_filt_prev' + F(n,:) - m )*(w1*S(:,n) + (1-w1)*T_prev);
    end        
end