function log_lkl = log_lkl_y(y, z, model_params, system_type)

% log_lkl = log_lkl_y(y, z, model_params, system_type)
%


% Extract the observed part of the data.
y = y((z==1));
n = length(y);

if n == 0
    % If no observation is available
    log_lkl = 0;
elseif strcmp(system_type, 'GC_content') == 1
    % Needed parameters:
    % Normal scaled inverse gamma parameters: N-IG(eta, delta, alpha, beta)
    alpha = model_params.inner_params.alpha;
    beta = model_params.inner_params.beta;
    delta = model_params.inner_params.delta;
    eta = model_params.inner_params.eta;
    
    log_lkl = -(n/2)*log(pi) + alpha*log(2*beta) ...
        + log(gamma(n/2 - floor(n/2) + alpha)) ...
        + sum(log(n/2 - floor(n/2) + alpha + (0:floor(n/2)-1)))...
        - log(gamma(alpha))- (n/2 + alpha)*log(sum((y - mean(y)).^2) ...
        + 2*beta + n*(mean(y)-eta)^2/(1 + n/delta)) - 0.5*log(1 + n/delta);    
    
elseif strcmp(system_type, 'Gaussian_mixture') == 1
    % Needed parameters:
    var_y = model_params.inner_params.var_y;
    mu_x = model_params.inner_params.mu_x;
    var_x = model_params.inner_params.var_x;
    A = ones(n,1);
    mu_y = A*mu_x;
    y = y(:);

    R = cholupdate(sqrt(var_y)*eye(n), A*sqrt(var_x));    
    z = R'\(y - mu_y);
     log_lkl = -0.5*(n*log(2*pi) + 2*sum(log(diag(R))) + z'*z);
%      disp([n log_lkl_1 log_lkl])

else
    error('Incorrect system type name');
end