% Main code for comparing the variances of the estimates in the expectation
% steps of the online-EM algorithms using the forward smoohthing method and
% the path space method:
%


clear all; close all; clc;
% Figure counter:
fc = 0;

%% System Specifications:
% Length of the observation data:
T = 100000;
% Number of possible models:
Q = 2;
% Initial distribution for the models
r = rand(Q, 1); mdl_init_dist = r/sum(r);
% Success probabilities
success_prob = 0.1*ones(Q,1);
% Matrix of transition probabilities for the models
% R = rand(Q);
% mdl_trans_mtx = R./repmat(sum(R, 2),1,Q);
mdl_trans_mtx = ones(Q)/(Q);

% Create a cell to store the model parameters:
model_params = cell(1,Q); 

% Create the parameters for the GC_content system:
eta = 2*randn(Q,1); delta = 0.5*rand(Q,1); alpha = 10; beta = 0.1;

% Create the cell array 'model_params', which contains in its i^th cell the
% corresponding model parameters.
for q = 1:Q

    seg_length_pmf_name = 'geometric_pdf';
    seg_length_pmf_params.success_prob = success_prob(q);
    seg_length_pmf_params.start_pnt = 1;

    model_params{q}.seg_length_pmf_name = seg_length_pmf_name;
    model_params{q}.seg_length_pmf_params = seg_length_pmf_params;

    data_log_lkl_pdf_name = 'log_lkl_GC_content';
    % data_log_lkl_pdf_name = 'log_lkl_Gaussian_mixture';
    model_params{q}.data_log_lkl_pdf_name = data_log_lkl_pdf_name;

    % Inner parameters for the GC_content system:
    inner_params.alpha = alpha;
    inner_params.beta = beta;
    inner_params.delta = delta(q);
    inner_params.eta = eta(q);
    
    model_params{q}.inner_params = inner_params;

end

%% Simulate the system
system_type = 'GC_content';
% system_type = 'Gaussian_mixture';
[simulated_system] = simul_chp_system(model_params, mdl_init_dist,...
    mdl_trans_mtx, T, system_type);
y = simulated_system.y;

fc = fc + 1; figure(fc);
plot(y); title('Observation data');
xlabel('t'); ylabel('y(t)');
drawnow;
pause;

%% Stability Performance of SMC-FS algorithm:

% Construct the initial values:
inputs.init_params.alpha = alpha;
inputs.init_params.beta = beta;
inputs.init_params.eta = eta;
inputs.init_params.delta = delta;
% Model specifications
inputs.data_log_lkl_pdf_name = 'log_lkl_GC_content';
inputs.system_type = 'GC_content';
inputs.seg_length_pmf_name = 'geometric_pdf';
% Common parameters
inputs.init_params.success_prob = success_prob;
inputs.init_params.mdl_trans_mtx = mdl_trans_mtx;
% Algorithm parameters:
inputs.Num_of_pars = 100;

Num_of_Runs = 100;

Num_of_S = Q^2 + 5*Q + 1;
SSS_FS = zeros(T, Num_of_S, Num_of_Runs);
SSS_PB = zeros(T, Num_of_S, Num_of_Runs);

% Run the algorithms:
for r = 1:Num_of_Runs
    disp(r);
    outputs = SMC_FS_and_PB_for_GC_content(y, inputs);    
    SSS_FS(:, :, r) = outputs.SS_FS;
    SSS_PB(:, :, r) = outputs.SS_PB;
end