% Main code for the to compare the forward smoothing and path based
% online-EM algorithms for changepoints
%

clear all; close all; clc;
% Figure counter:
fc = 0;

%% System Specifications:
% Length of the observation data:
T = 50000;
% Number of possible models:
Q = 2;
% Initial distribution for the models
r = rand(Q, 1); mdl_init_dist = r/sum(r);
% Success probabilities
success_prob = 0.1*ones(Q,1);
% Matrix of transition probabilities for the models
% R = rand(Q);
% mdl_trans_mtx = R./repmat(sum(R, 2),1,Q);
mdl_trans_mtx = ones(Q)/(Q);

% Create a cell to store the model parameters:
model_params = cell(1,Q); 

% Create the parameters for the GC_content system:
eta = 2*randn(Q,1); delta = 0.5*rand(Q,1); alpha = 10; beta = 0.1;

% Create the cell array 'model_params', which contains in its i^th cell the
% corresponding model parameters.
for q = 1:Q

    seg_length_pmf_name = 'geometric_pdf';
    seg_length_pmf_params.success_prob = success_prob(q);
    seg_length_pmf_params.start_pnt = 1;

    model_params{q}.seg_length_pmf_name = seg_length_pmf_name;
    model_params{q}.seg_length_pmf_params = seg_length_pmf_params;

    data_log_lkl_pdf_name = 'log_lkl_GC_content';
    % data_log_lkl_pdf_name = 'log_lkl_Gaussian_mixture';
    model_params{q}.data_log_lkl_pdf_name = data_log_lkl_pdf_name;

    % Inner parameters for the GC_content system:
    inner_params.alpha = alpha;
    inner_params.beta = beta;
    inner_params.delta = delta(q);
    inner_params.eta = eta(q);
    
    model_params{q}.inner_params = inner_params;

end

%% Simulate the system
system_type = 'GC_content';
% system_type = 'Gaussian_mixture';
[simulated_system] = simul_chp_system(model_params, mdl_init_dist,...
    mdl_trans_mtx, T, system_type);
y = simulated_system.y;

fc = fc + 1; figure(fc);
plot(y); title('Observation data');
xlabel('t'); ylabel('y(t)');
drawnow;
pause;
%% Parameter Estimation using on-line EM:

% Input parameters:
% % Initialize the unknown parameters:
alpha_init = 20; beta_init = 0.5; delta_init = rand(Q,1);
% % eta_est = randn(Q,1);
eta_init = eta + 0.5*randn(Q,1);

% Construct the initial values:
inputs.init_params.alpha = alpha_init;
inputs.init_params.beta = beta_init;
inputs.init_params.eta = eta_init;
inputs.init_params.delta = delta_init;

inputs.fix_params = [0 0 0 0 0 0]; %[suc prob, trans, alpha, beta, eta, delta]
inputs.data_log_lkl_pdf_name = 'log_lkl_GC_content';
inputs.system_type = 'GC_content';

% Common parameters
success_prob_init = 0.2*ones(Q,1);
% mdl_trans_mtx_init = (1/Q)*ones(Q);
mdl_trans_mtx_init = [[0.75 0.25];[0.25 0.75]];
% success_prob_init = success_prob;
% mdl_trans_mtx_init = mdl_trans_mtx;

inputs.init_params.success_prob = success_prob_init;
inputs.init_params.mdl_trans_mtx = mdl_trans_mtx_init;
inputs.seg_length_pmf_name = 'geometric_pdf';

% Algorithm parameters:
inputs.tilda = 0.8;
inputs.burn_in_per = 500;
inputs.Num_of_pars = 500;

% Number of EM iterations for offline EM
inputs.Num_of_EM_iters = 50;

% Vector of missing data indicators
z = ones(1,T);

% Number of MC runs
Num_of_MC_runs = 40;

% Initialise the arrays where the estimates over time are to be stored
Alpha_Est_FS = zeros(T, Num_of_MC_runs); 
Beta_Est_FS = zeros(T, Num_of_MC_runs);
Delta_Est_FS = zeros(Q, T, Num_of_MC_runs);
Eta_Est_FS = zeros(Q, T, Num_of_MC_runs);
Success_prob_Est_FS = zeros(Q, T, Num_of_MC_runs);
Mdl_trans_mtx_Est_FS = zeros(Q^2, T, Num_of_MC_runs);

Alpha_Est_PB = zeros(T, Num_of_MC_runs);
Beta_Est_PB = zeros(T, Num_of_MC_runs);
Delta_Est_PB = zeros(Q, T, Num_of_MC_runs);
Eta_Est_PB = zeros(Q, T, Num_of_MC_runs);
Success_prob_Est_PB = zeros(Q, T, Num_of_MC_runs);
Mdl_trans_mtx_Est_PB = zeros(Q^2, T, Num_of_MC_runs);

% Run the algorithms:
tic;
for r = 1:Num_of_MC_runs
    % online EM using the forward smoothing method:
    [online_EM_FS_outputs] = online_EM_FS_for_GC_content(y, inputs, z);
    Alpha_Est_FS(:, r) = online_EM_FS_outputs.Alpha_est;
    Beta_Est_FS(:, r) = online_EM_FS_outputs.Beta_est;
    Delta_Est_FS(:, :, r) = online_EM_FS_outputs.Delta_est;
    Eta_Est_FS(:, :, r) = online_EM_FS_outputs.Eta_est;
    Success_prob_Est_FS(:, :, r) = online_EM_FS_outputs.Success_prob_est;
    Mdl_trans_mtx_Est_FS(:, :, r) = online_EM_FS_outputs.Mdl_trans_mtx_est;
    
    % online EM using the path based method:
    [online_EM_PB_outputs] = online_EM_PB_for_GC_content(y, inputs, z);
    Alpha_Est_PB(:, r) = online_EM_PB_outputs.Alpha_est;
    Beta_Est_PB(:, r) = online_EM_PB_outputs.Beta_est;
    Delta_Est_PB(:, :, r) = online_EM_PB_outputs.Delta_est;
    Eta_Est_PB(:, :, r) = online_EM_PB_outputs.Eta_est;
    Success_prob_Est_PB(:, :, r) = online_EM_PB_outputs.Success_prob_est;
    Mdl_trans_mtx_Est_PB(:, :, r) = online_EM_PB_outputs.Mdl_trans_mtx_est;
end
