function [outputs] = offline_EM_for_GC_content(y, inputs, z)

% [outputs] = offline_EM_for_GC_content(y, inputs, z)
%
% This function performs the offline EM algorithm for changepoint models
% by using the forward smoothing method.
%


% Specify the algorithm parameters
T = length(y);
inputs.burn_in_per = T-1;
inputs.tilda = 1;

% Extract the initial values:
eta_est = inputs.init_params.eta;
Q = length(eta_est);
Num_of_EM_iters = inputs.Num_of_EM_iters;

% Initialize the arrays of parameter estimates over iterations:
Alpha_est = zeros(1, Num_of_EM_iters);
Beta_est = zeros(1, Num_of_EM_iters);
Delta_est = zeros(Q, Num_of_EM_iters);
Eta_est = zeros(Q, Num_of_EM_iters);
Success_prob_est = zeros(Q, Num_of_EM_iters);
Mdl_trans_mtx_est = zeros(Q^2, Num_of_EM_iters);

%% Recursion over time:
for em = 1:Num_of_EM_iters
    
    [one_step_outputs] = online_EM_for_GC_content(y, inputs, z);
    
    % Store the parameter estimates over time:
    alpha_est = one_step_outputs.Alpha_est(end);
    beta_est = one_step_outputs.Beta_est(end);
    delta_est = one_step_outputs.Delta_est(:,end);
    eta_est = one_step_outputs.Eta_est(:,end);
    success_prob_est = one_step_outputs.Success_prob_est(:,end);
    mdl_trans_mtx_est = reshape(one_step_outputs.Mdl_trans_mtx_est(:, end), Q, Q);
    
    % Store the parameter estimates over time:
    Alpha_est(em) = alpha_est;
    Beta_est(em) = beta_est;
    Delta_est(:,em) = delta_est;
    Eta_est(:,em) = eta_est;
    Success_prob_est(:,em) = success_prob_est;
    Mdl_trans_mtx_est(:,em) = mdl_trans_mtx_est(:);
    
    % Construct the new input parameters
    inputs.init_params.alpha = alpha_est;
    inputs.init_params.beta = beta_est;
    inputs.init_params.eta = eta_est;
    inputs.init_params.delta = delta_est;
    inputs.init_params.success_prob = success_prob_est;
    inputs.init_params.mdl_trans_mtx = mdl_trans_mtx_est;
 
    %%
    figure(10);
    subplot(321)
    plot(Alpha_est);
    title('\alpha estimate');
    
    subplot(322)
    plot(Beta_est);
    title('\beta estimate');
    
    subplot(323)
    plot(Eta_est');
    title('\eta estimate');
 
    subplot(324)
    plot(Delta_est');
    title('\delta estimate');
    
    subplot(325)
    plot(Success_prob_est');
    title('\gamma estimate');
    
    subplot(326);
    plot(Mdl_trans_mtx_est');
    title('P estimate');

drawnow;
    
end

%% Outputs
outputs.Alpha_est = Alpha_est;
outputs.Beta_est = Beta_est;
outputs.Delta_est = Delta_est;
outputs.Eta_est = Eta_est;
outputs.Success_prob_est = Success_prob_est;
outputs.Mdl_trans_mtx_est = Mdl_trans_mtx_est;