function [post_params] = post_dens_in_segment(prior_params,...
    system_type, y, z)

%  [post_params] = post_dens_in_segment(prior_params, system_type, y, z)

% Extract the observed part of the data vector if the data is possibly
% partly missing.
if nargin == 4
    y = y(z == 1);
end
n = length(y);

if strcmp(system_type, 'Gaussian_mixture') == 1
    % Gaussian Mixture
    mu_x = prior_params.mu_x;
    var_x = prior_params.var_x;
    var_y = prior_params.var_y;

    % Posterior dist. cinditioned on the current particle at time t
    var_post_x = 1/(1/var_x + n/var_y);
    mu_post_x = var_post_x*(sum(y)/var_y + mu_x/var_x);
    post_params.mu_post_x = mu_post_x;
    post_params.var_post_x = var_post_x;

elseif strcmp(system_type, 'GC_content') == 1
    alpha = prior_params.alpha;
    beta = prior_params.beta;
    eta = prior_params.eta;
    delta = prior_params.delta;
    
    % Posterior dist. conditioned on the current particle at time t
    alpha_post = alpha + n/2;
    if n==0
        beta_post = beta;
        eta_post = eta;
    else
        beta_post = beta + 0.5*sum((y - mean(y)).^2)...
            + (n*delta/(n+delta))*(mean(y)-eta)^2/2;
        eta_post = (delta*eta + n*mean(y))/(delta + n);
    end

    delta_post = delta + n;
    
    % Outputs
    post_params.alpha_post = alpha_post;
    post_params.beta_post = beta_post;
    post_params.eta_post = eta_post;
    post_params.delta_post = delta_post;
end