function Y = randgen(S, Siz, P)

% Y = RANDGEN(S, Siz, P) returns a weighted sample, using positive
% weights P. P is often a vector of probabilities but can be unnormalised.
% If P is absent we assume a uniform distribution
% 1
% Mark Scheme 2
% Example: Generate a random sequence of the characters ACGT, with
% replacement, according to specified probabilities.
% R = randgen('ACGT',48, [0.15 0.35 0.35 0.15])
% Example: Generate a random 3 by 3 matrix with independent entries from S = [1 2 5]
% according to specified weights.
% R = randgen([1 2 5], [3 3], [2 2 1])
% So on average there should be about twice as many one's as five's.
%


% Assume uniform  probabiltiy if not specified
if nargin == 2
    P = ones(size(S));
end
% If size is not specified print one
if nargin == 1
    Siz = [1 1];
    P = ones(size(S));
end
% If size is specified by a single number, assign the first dim. as 1
if length(Siz) == 1
    Siz = [1 Siz];
end
% normalize pmf if not and take the cumulative sum
P_cum = cumsum(P/sum(P));
Y = zeros(Siz(1), Siz(2));
for i = 1:Siz(1)
    for j = 1:Siz(2)
        u = rand;
        c = 1;
        while P_cum(c) < u 
            c = c+1;
        end
        Y(i,j) = S(c);
    end
end