function outputs = simul_data_for_one_segment(model_params, L, system_type)

% outputs = simul_data_for_one_segment(model_params, L, system_type)
%
% This function simulates latent and observable variables of a system of
% given type 'system_type', given the model parameters 'model_params' of
% the system. The observation length is determined by the input 'L'. This
% code can be extended by introducing new systems.
%
% The output structure 'outputs' contain the latent and the observable
% data, 'outputs.latent_data', and 'outputs.obs_data'.
%


if strcmp(system_type, 'GC_content') == 1
    % Needed parameters:
    % Normal scaled inverse gamma parameters: N-IG(eta, delta, alpha, beta)
    alpha = model_params.inner_params.alpha;
    beta = model_params.inner_params.beta;
    delta = model_params.inner_params.delta;
    eta = model_params.inner_params.eta;

    % Generate and store the latent data
    var_y = 1/(gamrnd(alpha, 1/beta));
    mu_y = eta + sqrt(var_y/delta)*randn;

    outputs.latent_data.mu_y = mu_y;
    outputs.latent_data.var_y = var_y;

    % Generate and store the observation:
    outputs.obs_data = sqrt(var_y)*randn(1, L) + mu_y;

elseif strcmp(system_type, 'Gaussian_mixture') == 1
    % Needed parameters:
    var_y = model_params.inner_params.var_y;
    mu_x = model_params.inner_params.mu_x;
    var_x = model_params.inner_params.var_x;

    % Generate and store the latent data
    x = mu_x + sqrt(var_x)*randn;
    outputs.latent_data.x = x;

    % Generate and store the observation:
    outputs.obs_data = x + sqrt(var_y)*randn(1,L);
else
    error('Incorrect system type name');
end
