function [ind, res_w] = stratified_res(log_w, res_param, type)

% [ind, res_w] = stratified_res(log_w, res_param, type)
%
% Stratified resampling
% Type 1: According to Fearnhead and Liu 2007, Online
% Inference for Multiple Changepoint Problems: Fixed cut-off value
% Type 2: According to Carpenter, J., Clifford, P. and Fearnhead, P. (1999)
% An improved particle filter for non-linear problems.
%


% Normalize candidate weights if they are not:
log_w = log_w - log_sum(log_w);
w = exp(log_w);
N = length(w);

if type == 1 %Number of resampled particles is randomly determined by a
    % cut-off value which is given as the parameter

    cut_off = res_param;
    % If the cut off is in log-scale
    if cut_off < 0
        log_cut_off = cut_off;
        cut_off = exp(cut_off);
    else
        log_cut_off = log(cut_off);
    end

    u = cut_off*rand;

    ind = zeros(1,N);
    res_w = zeros(1, N);

    ind_count = 0;
    for n = 1:N
        if w(n) >= cut_off
            ind_count = ind_count + 1;
            ind(ind_count) = n;
            res_w(ind_count) = log_w(n);
        else
            u = u - w(n);
            if u <= 0
                ind_count = ind_count + 1;
                ind(ind_count) = n;
                res_w(ind_count) = log_cut_off;
                u = u + cut_off;
            end
        end
    end
    ind = ind(1:ind_count);
    res_w = res_w(1:ind_count);

elseif type == 2 % Number of resampled particles is given as the parameter

    M = res_param;
    if N >= M
        %% find the alpha which satisfies sum_{n}(min(1, alpha*w_{n} )) = M
        [log_w_sorted, ind_sort] = sort(log_w, 'descend');
        A = 0;
        while A < M
            A = A + 1;
            alpha = log_sum(log_w_sorted(A+1:N))-log(M-A);
            if sum(log_w > alpha) == A
                break;
            end
        end
        
        if A < M

            % Resample weights greter then alpha
            v = 1:N;
            ind(1:A) = v(log_w > alpha);
            res_w(1:A) = log_w(log_w > alpha);

            % Get the small weights and their indices
            ind_small = v(log_w <= alpha);
            log_w_small = log_w(log_w <= alpha);
            % Normalize
            w_small = exp(log_w_small - log_sum(log_w_small));
            W_small = cumsum(w_small);

            r = rand/(M-A);
            for m = 1:M-A
                u = r + (m-1)/(M-A);
                i = sum(u > W_small) + 1;
                ind(A + m) = ind_small(i);
                res_w(A + m) = alpha;
            end
        else
            res_w = log_w(ind_sort(1:M));
            ind = ind_sort(1:M);
        end
    else
        res_w = log_w;
        ind = 1:N;
    end
end