% Main code for online EM for the the DNA GC content data


%% Load data:
clear all; clc; close all;
load GC_data_chr2_part1;

Y1 = GC_data.GC_cont;
% Find the missing data intervals
Z1 = (Y1 > 0);

load GC_data_chr2_part2;
Y2 = GC_data.GC_cont;
% Find the missing data intervals
Z2 = (Y2 > 0);

Y = [Y1 Y2];
Z = [Z1 Z2];

Num_of_repeats = 10;
y = repmat(Y, 1, Num_of_repeats);
z = repmat(Z, 1, Num_of_repeats);
subplot(211);
plot(y);
subplot(212);
plot(z);
pause;

%% Inputs:
% Initial values:
Q = 2;

% % GC content
alpha_init = 5;
beta_init = 0.05;
% eta_init = [0.39; 0.44; 0.48; 0.53];
eta_init = [0.40; 0.50];
% eta_init = mean(y);
delta_init = ones(Q, 1);

% Common parameters
success_prob_init = 0.1*ones(Q,1);
mdl_trans_mtx_init = (1/Q)*ones(Q);
% success_prob_init = offline_outputs.Success_prob_est(:,end);
% mdl_trans_mtx_init = offline_outputs.mdl_trans_mtx_est;

%% Construct the initial values:
inputs.init_params.alpha = alpha_init;
inputs.init_params.beta = beta_init;
inputs.init_params.delta = delta_init;
inputs.init_params.eta = eta_init;
inputs.init_params.success_prob = success_prob_init;
inputs.init_params.mdl_trans_mtx = mdl_trans_mtx_init;
inputs.seg_length_pmf_name = 'geometric_pdf';

inputs.fix_params = [0 0 0 0 0 0]; %[suc prob, trans, alpha, beta, eta, delta]
inputs.data_log_lkl_pdf_name = 'log_lkl_GC_content';
inputs.system_type = 'GC_content';

% Algorithm parameters:
inputs.tilda = 0.80;
inputs.burn_in_per = 1000;
inputs.weight_thr = -6;
inputs.num_thr = 100;
% for off-line EM
inputs.Num_of_EM_iters = 30;

%% Choose the correponding function

% On-line EM for GC_content:
tic;
[online_outputs] = online_EM_for_GC_content(y, inputs, z);
toc;

% Off-line EM for GC_content
% tic;
% [offline_outputs] = offline_EM_for_GC_content(y, inputs, z);
% s_save = 'offline_EM_GC_DNA_chr1_part_2_weight_thr_6_means_unfixed';
% save([s_save date]);
% toc;